#!/bin/sh

# NAME:
#	shutdown-all - shutdown all hosts on UPS
#
# SYNOPSIS:
#	shutdown-all [start] ["host" ...]
#	shutdown-all powerdown "host"
#
# DESCRIPTION:
#	This script is run from the lowbat state by 'apcmon'.
#	I have:
#
#.nf
#		/etc/apc/lowbat -> rc.sh
#		/etc/apc/lowbat.d/S60shutdown-all -> ../shutdown-all.sh
#		/etc/apc/lowbat.d/shutdown-all.rc
#
#.fi
#
#	It attempts to shutdown all the hosts protected by the UPS
#	being monitored.
#
#	Normally the list of hosts to shutdown comes from
#	shutdown-all.rc which can also override functions to adapt to
#	an OS other than 4.4BSD
#
#	shutdown-all.rc is expected to set "SHUTDOWN_HOST_LIST"
#	and "SHUTDOWN_HOST_USER" (a user who has an ssh-agent running
#	that will allow login to all the hosts as root).
#	
#	For each "host" we run a sub-shell with a timeout to try to
#	shut it down if it is up.
#
#	For the sub-shell we run ourselves with first arg of
#	'powerdown' to indicate we are shutting down one host.
#
#	If the "SHUTDOWN_HOST_LIST" includes '.PAUSE' we will pause
#	for "SHUTDOWN_HOST_PAUSE" seconds (default 20s) to allow the
#	preceeding "host" to get a headstart on those that follow.
#	This is useful when the host list includes switches which need
#	to be shutdown last once everything else has been told to.
#	If the list contains '.WAIT' we will 'wait' for all preceding
#	jobs to complete - which should not take more than
#	"SHUTDOWN_HOST_TIMEOUT".
#
# NOTE:
#	The 'ping' and 'timeout' semantics are OS specific, you may
#	need to modify/override via 'shutdown-all.rc'
#	

# RCSid:
#	$Id: shutdown-all.sh,v 1.6 2023/04/30 00:10:05 sjg Exp $
#
#	@(#) Copyright (c) 1998-2023 Simon J. Gerraty
#
#	This file is provided in the hope that it will
#	be of use.  There is absolutely NO WARRANTY.
#	Permission to copy, redistribute or otherwise
#	use this file is hereby granted provided that 
#	the above copyright notice and this notice are
#	left intact. 
#      
#	Please send copies of changes and bug-fixes to:
#	sjg@crufty.net
#

MYNAME=shutdown-all

opt_L=${opt_L:-local6}

export PATH=$PATH:/share/bin

. debug.sh
DebugOn apcmon_shutown $MYNAME
. source.sh

LOGGER_TAG="$MYNAME[$$]"

# you might need to override ping1 and _timeout
# These are NetBSD/FreeBSD compatible

# ping1 host
# send one ping to "host" and don't wait too long for response
ping1() {
    ping -c 1 -W 500 $1 > /dev/null 2>&1
}

# timeout "timeout" "command line"
# run "command line" but kill it after "timeout"
_timeout() {
    timeout -k 5s "$@"
}

error() {
    echo "ERROR: $@" >&2
    exit 1
}

# we need SSH_AUTH_SOCK for a user with ability to
# login as root.
if [ -z "$SSH_AUTH_SOCK" ]; then
    _source ssh-agent.sh
    export SSH_AUTH_SOCK=`USER=${SHUTDOWN_HOST_USER:-sjg}; ssh_agent_find; echo $SSH_AUTH_SOCK`
    [ -z "$SSH_AUTH_SOCK" ] && error need SSH_AUTH_SOCK
fi

trap "" 1

powerdown1() {
    if ping1 $1; then
	logger -t "$LOGGER_TAG" -p $opt_L.notice Telling $1 to power down ...
	_timeout ${2:-${SHUTDOWN_HOST_TIMEOUT:-4m}} $0 powerdown $1
    else
	logger -t "$LOGGER_TAG" -p $opt_L.notice host: $1: down/asleep
    fi
}

# we will be killed after a few minutes
powerdown_host() {
    $SHUTDOWN_ECHO ${SSH:-ssh} -l root $1 shutdown -p now || true
    while ping1 $1
    do
	sleep ${SHUTDOWN_HOST_PING_SLEEP:-8}
    done
    logger -t "$LOGGER_TAG" -p $opt_L.notice host: $1: down
}

powerdown_all() {
    for h in ${@:-$SHUTDOWN_HOST_LIST}
    do
        case "$h" in
	.PAUSE) sleep ${SHUTDOWN_HOST_PAUSE:-20}; continue;;
	.WAIT) wait; continue;;
	esac
	powerdown1 $h &
    done
}

Mydir=${Mydir:-`dirname $0`}
dot $Mydir/$MYNAME.rc

case "$1" in
powerdown) powerdown_host $2;;
start) shift; powerdown_all "$@";;
*) powerdown_all "$@";;
esac
